package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.City;
import my.test.sakila.model.Country;

/**
 * Backing bean for City entities.
 * <p>
 * This class provides CRUD functionality for all City entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class CityBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving City entities
	 */

	private Short id;

	public Short getId() {
		return this.id;
	}

	public void setId(Short id) {
		this.id = id;
	}

	private City city;

	public City getCity() {
		return this.city;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.city = this.search;
		} else {
			this.city = this.entityManager.find(City.class, getId());
		}
	}

	/*
	 * Support updating and deleting City entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.city);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.city);
				return "view?faces-redirect=true&id=" + this.city.getCityId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(City.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching City entities with pagination
	 */

	private int page;
	private long count;
	private List<City> pageItems;
	
	private City search = new City();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public City getSearch() {
		return this.search;
	}

	public void setSearch(City search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<City> root = countCriteria.from(City.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<City> criteria = builder.createQuery(City.class);
		root = criteria.from(City.class);
		TypedQuery<City> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<City> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		Country country = this.search.getCountry();
		if (country != null && country.getCountryId() != null) {
			predicatesList.add(builder.equal(root.get("country"), country));
		}
		String city = this.search.getCity();
		if (city != null && !"".equals(city)) {
			predicatesList.add(builder.like(root.<String>get("city"), '%' + city + '%'));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<City> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back City entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<City> getAll() {

		CriteriaQuery<City> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(City.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(City.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return CityBean.this.entityManager.find(City.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((City) value).getCityId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private City add = new City();

	public City getAdd() {
		return this.add;
	}

	public City getAdded() {
		City added = this.add;
		this.add = new City();
		return added;
	}
}